import type { ObjectFixedIntersectionToObject, CommonAdapterPool, CommonAdapterOptions, AdapterOperations, AdapterConnection, AdapterImplementation, StoreApi, PerformanceTracerLike, SplitNestedPathMethod, ReadModelLedger, JsonMap, SearchCondition, UpdateCondition, ObjectFixedKeys, OmitObject, EventThreadData } from '@resolve-js/readmodel-base';
import type PGLib from 'pg';
export * from '@resolve-js/readmodel-base';
export declare type LibDependencies = {
    Postgres: typeof PGLib.Client;
};
export declare type InlineLedgerRunQueryMethod = (querySQL: string, passthroughRuntimeErrors?: boolean) => Promise<Array<object>>;
export declare type FullJitterMethod = (retries: number) => Promise<void>;
export declare type MakeNestedPathMethod = (nestedPath: Array<string>) => string;
export declare type EscapeableMethod = (str: string) => string;
export declare type InlineLedgerForceStopMethod = (pool: AdapterPool, readModelName: string) => Promise<void>;
export declare type BuildUpsertDocumentMethod = (searchExpression: Parameters<StoreApi<CommonAdapterPool>['update']>[3], updateExpression: Parameters<StoreApi<CommonAdapterPool>['update']>[4], splitNestedPath: SplitNestedPathMethod) => JsonMap;
export declare type RowLike = JsonMap;
export declare type MarshalledRowLike = RowLike & {
    marshalled: 'marshalled';
};
export declare type ConvertResultRowMethod = (inputRow: MarshalledRowLike, fieldList: Parameters<StoreApi<CommonAdapterPool>['find']>[4] | null) => RowLike;
export declare type SearchToWhereExpressionMethod = (expression: SearchCondition, escapeId: EscapeableMethod, escapeStr: EscapeableMethod, makeNestedPath: MakeNestedPathMethod, splitNestedPath: SplitNestedPathMethod) => string;
export declare type UpdateToSetExpressionMethod = (expression: UpdateCondition, escapeId: EscapeableMethod, escapeStr: EscapeableMethod, makeNestedPath: MakeNestedPathMethod, splitNestedPath: SplitNestedPathMethod) => string;
export interface PassthroughErrorInstance extends Error {
    name: string;
    isRetryable: boolean;
}
export declare type PassthroughErrorLike = Error & {
    code: string | number;
    stack: string;
};
export declare type PassthroughErrorFactory = {
    new (isRetryable: boolean): PassthroughErrorInstance;
} & {
    isRetryablePassthroughError: (error: PassthroughErrorLike) => boolean;
    isRegularFatalPassthroughError: (error: PassthroughErrorLike) => boolean;
    isRuntimeFatalPassthroughError: (error: PassthroughErrorLike) => boolean;
    isPassthroughError: (error: PassthroughErrorLike, includeRuntimeErrors: boolean) => boolean;
    maybeThrowPassthroughError: (error: PassthroughErrorLike, includeRuntimeErrors: boolean) => void;
};
export declare type GenerateGuidMethod = (...args: any) => string;
export declare type DropReadModelMethod = (pool: AdapterPool, readModelName: string) => Promise<void>;
export declare type BuildMode = 'plv8-internal' | 'plv8-external' | 'plv8' | 'nodejs' | 'auto';
export declare type AdapterOptions = CommonAdapterOptions & {
    databaseName: string;
    tablePrefix?: string;
    buildMode?: BuildMode;
    useSqs?: boolean;
} & PGLib.ConnectionConfig;
export declare type MaybeInitMethod = (pool: AdapterPool) => Promise<void>;
export declare type MakeSqlQueryMethodTargetParameters<T extends keyof CurrentStoreApi> = Parameters<CurrentStoreApi[T]> extends [infer _, infer __, ...infer Args] ? Args : never;
export declare type MakeSqlQueryMethod = <T extends keyof CurrentStoreApi>(methods: AdapterPool, readModelName: string, operation: T, ...args: MakeSqlQueryMethodTargetParameters<T>) => string;
export declare type InternalMethods = {
    inlineLedgerForceStop: InlineLedgerForceStopMethod;
    buildUpsertDocument: BuildUpsertDocumentMethod;
    convertResultRow: ConvertResultRowMethod;
    searchToWhereExpression: SearchToWhereExpressionMethod;
    updateToSetExpression: UpdateToSetExpressionMethod;
    makeNestedPath: MakeNestedPathMethod;
    makeSqlQuery: MakeSqlQueryMethod;
    PassthroughError: PassthroughErrorFactory;
    generateGuid: GenerateGuidMethod;
    dropReadModel: DropReadModelMethod;
    escapeId: EscapeableMethod;
    escapeStr: EscapeableMethod;
    maybeInit: MaybeInitMethod;
};
export declare type ArrayOrSingleOrNull<T> = Array<T> | T | null;
export declare type UpdateFieldDescriptor = {
    [K in ObjectFixedKeys<ObjectFixedIntersectionToObject<UpdateCondition>>]: ObjectFixedIntersectionToObject<UpdateCondition>[K][string] | null;
} & {
    key: string;
    nestedKey: string[];
    baseName: string;
    selectedOperation: ObjectFixedKeys<UpdateCondition> | null;
    children: Map<string, UpdateFieldDescriptor>;
    operations?: ArrayOrSingleOrNull<{
        operationName: ObjectFixedKeys<UpdateCondition> | null;
        fieldValue: UpdateFieldDescriptor[ObjectFixedKeys<UpdateCondition>];
        nestedPath?: string[];
    }>;
};
export declare type AdapterPool = CommonAdapterPool & {
    inlineLedgerRunQuery: InlineLedgerRunQueryMethod;
    performanceTracer: PerformanceTracerLike;
    tablePrefix: string;
    schemaName: string;
    makeNestedPath: MakeNestedPathMethod;
    activePassthrough: boolean;
    connection: InstanceType<LibDependencies['Postgres']>;
    buildMode: BuildMode;
    useSqs: boolean;
} & {
    [K in keyof AdapterOperations<CommonAdapterPool>]: AdapterOperations<AdapterPool>[K];
} & {
    [K in keyof StoreApi<CommonAdapterPool>]: StoreApi<AdapterPool>[K];
} & InternalMethods;
export declare type CurrentAdapterConnection = AdapterConnection<AdapterPool, OmitObject<AdapterOptions, CommonAdapterPool>>;
export declare type ExternalMethods = {
    [K in keyof AdapterOperations<CommonAdapterPool>]: AdapterPool[K];
};
export declare type CurrentStoreApi = {
    [K in keyof StoreApi<CommonAdapterPool>]: AdapterPool[K];
};
export declare type ConnectionDependencies = LibDependencies & InternalMethods & ExternalMethods & CurrentStoreApi;
export declare type CurrentConnectMethod = (imports: ConnectionDependencies, ...args: Parameters<CurrentAdapterConnection['connect']>) => ReturnType<CurrentAdapterConnection['connect']>;
export declare type CurrentDisconnectMethod = CurrentAdapterConnection['disconnect'];
export declare type CurrentAdapterImplementation = AdapterImplementation<AdapterPool, AdapterOptions>;
export declare type AdminOptions = PGLib.ConnectionConfig & {
    databaseName: string;
    userLogin: string;
};
export declare type ReadModelProcedureLedger = ReadModelLedger & {
    IsProcedural: boolean;
};
export declare type ProcedureResult = {
    appliedEventsThreadData: Array<EventThreadData>;
    status: 'OK_ALL' | 'OK_PARTIAL' | 'CUSTOM_ERROR' | 'DEPENDENCY_ERROR';
    successEvent: Event | null;
    failureEvent: Event | null;
    failureError: Error | null;
    appliedCount: number;
};
export declare type BoundResourceMethod = (options: AdminOptions) => Promise<void>;
export declare type UnboundResourceMethod = (pool: AdminPool, options: AdminOptions) => Promise<void>;
export declare type AdminPool = {
    connect: CurrentAdapterImplementation['connect'];
    disconnect: CurrentAdapterImplementation['disconnect'];
    escapeStr: EscapeableMethod;
    escapeId: EscapeableMethod;
    createResource: BoundResourceMethod;
    destroyResource: BoundResourceMethod;
};
